unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, AppEvnts, DirectDraw, DDUtil;

type
  TForm1 = class(TForm)
    ApplicationEvents1: TApplicationEvents;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure ApplicationEvents1Restore(Sender: TObject);
    procedure ApplicationEvents1Deactivate(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
  private
    { Private declarations }
    FDirectDraw : IDirectDraw7;
    FPrimarySurface   : IDirectDrawSurface7;
    FBackGround       : IDirectDrawSurface7;
    FImageSurface     : IDirectDrawSurface7;
    FTransparentImage : IDirectDrawSurface7;
    MouseX, MouseY    : Integer;
    procedure ErrorOut(hRet : HRESULT);
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

procedure TForm1.ErrorOut(hRet: HRESULT);
begin
 MessageBox(0, PChar(DDErrorString(hRet)), '', MB_OK or MB_ICONSTOP);
end;

procedure TForm1.FormCreate(Sender: TObject);
var
 hRet : HRESULT;
 SurfaceDesc : TDDSurfaceDesc2;
 fcaps : TDDSCaps2;
begin
 FPrimarySurface := nil;
 FDirectDraw := nil;
 FBackGround :=nil;
 FImageSurface :=nil;
 FTransparentImage := nil;
 MouseX:=0;
 MouseY:=0;

 hRet := DirectDrawCreateEx (nil, FDirectDraw, IDirectDraw7, nil);
 if hRet <> DD_OK then
  begin
   ErrorOut(hRet);
   Close;
   Exit;
  end;

 hRet := FDirectDraw.SetCooperativeLevel(Handle, DDSCL_FULLSCREEN or DDSCL_EXCLUSIVE);
 if hRet <> DD_OK then
  begin
   ErrorOut(hRet);
   Close;
   Exit;
  end;

 hRet := FDirectDraw.SetDisplayMode (800, 600, 16, 0, 0);
 if hRet <> DD_OK then
  begin
   ErrorOut(hRet);
   Close;
   Exit;
  end;

 FillChar (SurfaceDesc, SizeOf(SurfaceDesc), 0);
 SurfaceDesc.dwSize := SizeOf(SurfaceDesc);
 SurfaceDesc.dwFlags := DDSD_CAPS or DDSD_BACKBUFFERCOUNT;
 SurfaceDesc.ddsCaps.dwCaps := DDSCAPS_PRIMARYSURFACE or DDSCAPS_FLIP or DDSCAPS_COMPLEX;
 SurfaceDesc.dwBackBufferCount := 1;
 hRet := FDirectDraw.CreateSurface(SurfaceDesc, FPrimarySurface, nil);
 if hRet <> DD_OK then
  begin
   ErrorOut(hRet);
   Close;
   Exit;
  end;

 FImageSurface := DDLoadBitmap(FDirectDraw, '1.bmp', 0, 0);
 if FImageSurface = nil then
  begin
   ErrorOut(hRet);
   Close;
   Exit;
  end;

 FillChar(fcaps, SizeOf(fcaps), 0);
 fcaps.dwCaps := DDSCAPS_BACKBUFFER;
 hRet := FPrimarySurface.GetAttachedSurface(fcaps, FBackGround);
 if hRet <> DD_OK then
  begin
   ErrorOut(hRet);
   Close;
   Exit;
  end;

 FTransparentImage := DDLoadBitmap(FDirectDraw, 'bart.bmp', 0, 0);
 if FTransparentImage = nil then
  begin
   ErrorOut(hRet);
   Close;
   Exit;
  end;

 DDSetColorKey (FTransparentImage, RGB(255, 0, 255));
 if hRet <> DD_OK then
  begin
   ErrorOut(hRet);
  end;
 ShowCursor(false);
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
 if Assigned(FTransparentImage) then FTransparentImage := nil;
 if Assigned(FImageSurface) then FImageSurface := nil;
 if Assigned(FBackGround) then FBackGround := nil;
 if Assigned(FPrimarySurface) then FPrimarySurface := nil;
 if Assigned(FDirectDraw) then FDirectDraw := nil;
 ShowCursor(true);
end;

procedure TForm1.FormPaint(Sender: TObject);
var
 hRet : HRESULT;
 bltfx : TDDBLTFX;
begin
 ZeroMemory(@bltfx, SizeOf(bltfx));
 bltfx.dwSize := sizeof(bltfx);
 bltfx.dwFillColor := 0;
 while True do
  begin
   hRet := FBackGround.Blt(nil, nil, nil, DDBLT_COLORFILL or DDBLT_WAIT, @bltfx);
   if hRet = DDERR_SURFACELOST then
    begin
     hRet := FPrimarySurface._Restore;
     FImageSurface := DDLoadBitmap(FDirectDraw, '1.bmp', 0, 0);
     FTransparentImage := DDLoadBitmap(FDirectDraw, 'bart.bmp', 0, 0);
     if hRet <> DD_OK then exit;
    end
   else Break;
  end;

 FBackGround.BltFast (175, 75, FImageSurface, nil, DDBLTFAST_WAIT);
 FBackGround.BltFast(mouseX-20, mouseY-47, FTransparentImage, nil,
                  DDBLTFAST_WAIT or DDBLTFAST_SRCCOLORKEY);
 FPrimarySurface.Flip(nil, DDFLIP_WAIT);
end;

procedure TForm1.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
 if Key=VK_ESCAPE then Close;
end;

procedure TForm1.ApplicationEvents1Restore(Sender: TObject);
begin
 WindowState := wsMaximized;
end;

procedure TForm1.ApplicationEvents1Deactivate(Sender: TObject);
begin
 Application.Minimize;
end;

procedure TForm1.FormMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
 mouseX := X;
 mouseY := Y;
 FormPaint (nil);
end;

end.
